package ij.io;
import java.io.*;
import java.net.*;

/** ImageJ uses this class loader to load plugins from the plugins
	folder. It's based on the FileClassLoader from
	"Java Class Libraries: Second Edition, Vol. 1"
	(http://java.sun.com/docs/books/chanlee/second_edition/vol1/).
*/
public class PluginClassLoader extends ClassLoader {
    String path;

    public PluginClassLoader(String path) {
        this.path = path;
    }

    // Loads the bytes from file 
    Class loadIt(String classname) throws ClassNotFoundException {
        String filename = classname.replace('.','/');
        filename += ".class";
        File fullname = new File(path, filename);
		//ij.IJ.write("loadIt: " + fullname);
        try { // read the byte codes
            InputStream is = new FileInputStream(fullname);
            int bufsize = (int)fullname.length();
            byte buf[] = new byte[bufsize];
            is.read(buf, 0, bufsize);
            is.close();
            return defineClass(classname, buf, 0, buf.length);
        } catch (Exception e) {
            throw new ClassNotFoundException(classname);
        }
    }

    protected synchronized Class loadClass(String name, boolean resolve)
            throws ClassNotFoundException {
		//ij.IJ.write("loadClass: "+name+" "+resolve);
        Class c = findLoadedClass(name); // try to find it from the cache
        //ij.IJ.write("  "+((c == null) ? "not " : " ") +"in cache: "+name);

        // Not in cache
        if (c == null) {
            // See if it can be loaded by system class loader
            try {
                // No need to call resolveClass() on the result
                // because findSystemClass() loads and links the class.
                return findSystemClass(name);
            } catch (ClassNotFoundException e) {
            }
            // Try to get it from file
            c = loadIt(name);
        }

        // Link class if asked to do so
        if (c != null && resolve) {
	    //ij.IJ.write("  resolving: " + name);
            resolveClass(c);
        }
        return c;
    }

}
